package com.daikin.wechat.api;

import com.daikin.wechat.config.MyElasticsearchClient;
import org.apache.log4j.Logger;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.text.Text;
import org.elasticsearch.common.unit.Fuzziness;
import org.elasticsearch.common.unit.TimeValue;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightBuilder;
import org.elasticsearch.search.sort.SortBuilder;
import org.elasticsearch.search.sort.SortBuilders;
import org.elasticsearch.search.sort.SortOrder;
import org.junit.jupiter.api.Test;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

public class SearchApi {

    public static Logger logger = Logger.getLogger(SearchApi.class);

    @Test
    public void searchApiSync(String[] indices, String keyWord) {

        try (RestHighLevelClient client = MyElasticsearchClient.getInstance()) {

            QueryBuilder matchQueryBuilder = QueryBuilders.multiMatchQuery(keyWord)
                    .fuzziness(Fuzziness.AUTO);

            // 设置高亮,使用默认的highlighter高亮器
            HighlightBuilder highlightBuilder = createHighlightBuilder("custom_audiences.name", "geo_locations.countries");

            SearchSourceBuilder builder = new SearchSourceBuilder()
                    .query(matchQueryBuilder)
                    .from(0)
                    .size(2)
                    .highlighter(highlightBuilder)
                    .timeout(new TimeValue(60, TimeUnit.SECONDS));

            SearchRequest request = new SearchRequest(indices)
                    .source(builder);

            SearchResponse response = client.search(request, RequestOptions.DEFAULT);

            System.out.println("=== HTTP Request ===");
            System.out.println("status: " + response.status());
            System.out.println("took: " + response.getTook());
            System.out.println("timed_out: " + response.isTimedOut());

            System.out.println("\n=== Hits ===");
            SearchHits hits = response.getHits();
            System.out.println("total_hits: " + hits.getTotalHits());

            List<Map<String,Object>> list = getHitList(hits);

            for (SearchHit hit : hits.getHits()) {
                System.out.println("\n=== Documents ===");
                System.out.println("index: " + hit.getIndex());
                System.out.println("id: " + hit.getId());
                System.out.println("source: " + hit.getSourceAsString());
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 构造高亮器
     */
    private HighlightBuilder createHighlightBuilder(String... fieldNames){
        // 设置高亮,使用默认的highlighter高亮器
        HighlightBuilder highlightBuilder = new HighlightBuilder();

        // 设置高亮字段
        for (String fieldName: fieldNames) highlightBuilder.field(fieldName);

        highlightBuilder.preTags("<span style='color:red'>")
                .postTags("</span>");

        return highlightBuilder;
    }

    /**
     * 处理高亮结果
     */
    private List<Map<String,Object>> getHitList(SearchHits hits){
        List<Map<String,Object>> list = new ArrayList<>();
        Map<String,Object> map;
        for(SearchHit searchHit : hits){
            map = new HashMap<>();
            // 处理源数据
            map.put("source",searchHit.getSourceAsMap());
            // 处理高亮数据
            Map<String,Object> hitMap = new HashMap<>();
            searchHit.getHighlightFields().forEach((k,v) -> {
                String hight = "";
                for(Text text : v.getFragments()) hight += text.string();
                hitMap.put(v.getName(),hight);
            });
            map.put("highlight",hitMap);
            list.add(map);
        }
        return list;
    }

    /**
     * 　* @Description match all 查询
     * 　* @param
     * 　* @return
     * 　* @throws
     */
    @Test
    public void demo5() {

        QueryBuilder matchAllQueryBuilder = QueryBuilders
                .matchAllQuery();

        SearchSourceBuilder builder = new SearchSourceBuilder()
                .query(matchAllQueryBuilder)
                .timeout(new TimeValue(60, TimeUnit.SECONDS));

        excuteTestSearch(builder);
    }

    /**
     * 　* @Description sort 排序
     *
     * 　* @return
     * 　* @throws
     */
    @Test
    public void demo1() {

        QueryBuilder matchQueryBuilder = QueryBuilders
                .matchQuery("make", "honda");

        SortBuilder sortBuilder = SortBuilders.fieldSort("price").order(SortOrder.DESC);

        SearchSourceBuilder builder = new SearchSourceBuilder()
                .query(matchQueryBuilder)
                .sort(sortBuilder)
                .timeout(new TimeValue(60, TimeUnit.SECONDS));

        excuteTestSearch(builder);
    }

    /**
     * 　* @Description 分页查询
     *
     * 　* @return
     * 　* @throws
     */
    @Test
    public void demo2() {

        QueryBuilder matchQueryBuilder = QueryBuilders
                .matchAllQuery();

        SearchSourceBuilder builder = new SearchSourceBuilder()
                .query(matchQueryBuilder)
                .from(0)
                .size(2)
                .timeout(new TimeValue(60, TimeUnit.SECONDS));
        excuteTestSearch(builder);
    }

    /**
     * 　* @Description range 查询
     *
     * @author lee jun
     */
    @Test
    public void demo3() {

        QueryBuilder rangeQueryBuilder = QueryBuilders
                .rangeQuery("price").gte(30000);

        SearchSourceBuilder builder = new SearchSourceBuilder()
                .query(rangeQueryBuilder)
                .timeout(new TimeValue(60, TimeUnit.SECONDS));

        excuteTestSearch(builder);
    }

    /**
     * 　* @Description term 查询
     * 　* @throws
     */
    @Test
    public void demo4() {

        QueryBuilder termQueryBuilder = QueryBuilders
                .termQuery("color", "red");

        SearchSourceBuilder builder = new SearchSourceBuilder()
                .query(termQueryBuilder)
                .timeout(new TimeValue(60, TimeUnit.SECONDS));

        excuteTestSearch(builder);
    }

    public void excuteTestSearch(SearchSourceBuilder builder) {
        try (RestHighLevelClient client = MyElasticsearchClient.getInstance()) {

            SearchRequest request = new SearchRequest("test")
                    .source(builder);

            SearchResponse response = client.search(request, RequestOptions.DEFAULT);

            System.out.println("=== HTTP Request ===");
            System.out.println("status: " + response.status());
            System.out.println("took: " + response.getTook());
            System.out.println("timed_out: " + response.isTimedOut());

            System.out.println("\n=== Hits ===");
            SearchHits hits = response.getHits();
            System.out.println("total_hits: " + hits.getTotalHits());

            for (SearchHit hit : hits.getHits()) {
                System.out.println("\n=== Documents ===");
                System.out.println("index: " + hit.getIndex());
                System.out.println("id: " + hit.getId());
                System.out.println("source: " + hit.getSourceAsString());
            }
            System.out.println("\n=== Builder ===");
            System.out.println(builder);

        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }



    public static void main(String[] args) throws InterruptedException {
        SearchApi searchApi = new SearchApi();
        searchApi.searchApiSync(new String[]{"test","contact"},"国家");
    }
}
